@@ -127,23 +127,46 @@ class DatasetStateManager:
         bucket, file_path = parse_s3_url(s3_url)
         # load data from s3 as byte steam
         data_stream = io.BytesIO()
-        self.s3_client.download_fileobj(
-            bucket,
-            file_path,
-            data_stream
-        )
-        data_stream.seek(0)
+        try:
+            self.s3_client.download_fileobj(
+                bucket,
+                file_path,
+                data_stream
+            )
+            data_stream.seek(0)
+
+        except botocore_clienterror as exception:
+            if exception.response['Error']['Code'] == "404":
+                # The object does not exist.
+                print((
+                    "Unable to find s3 object. Skipping the task, "
+                    "the following exception was found = {}".format(exception)
+                ))
+            else:
+                raise
+
         return data_stream
 
     def upload_s3_blob(self, blob: io.BytesIO, s3_url: str):
         """download blob from s3"""
         # parse s3 url -> bucket, file_path
         bucket, file_path = parse_s3_url(s3_url)
-        self.s3_client.upload_fileobj(
-            blob, bucket, file_path
-        )
+        try:
+            # upload to s3
+            self.s3_client.upload_fileobj(
+                blob, bucket, file_path
+            )
+        except botocore_clienterror as exception:
+            if exception.response['Error']['Code'] == "404":
+                # The object does not exist.
+                print((
+                    "Unable to find s3 object. Skipping the task, "
+                    "the following exception was found = {}".format(exception)
+                ))
+            else:
+                raise
 
-    def execute_s3_versioning(self, data_stream: io.BytesIO, dst_url: str):
+    def execute_s3_versioning(self, json_data: dict, dst_url: str):
         """
             Streams data from source s3 url to a destination s3 url
 
@@ -151,31 +174,32 @@ class DatasetStateManager:
                 src_url (str): source s3 url
                 dst_url (str): destination s3 url
         """
-        try:
-            self.upload_s3_blob(
-                blob=data_stream,
-                s3_url=dst_url
-            )
-        except botocore_clienterror as exception:
-            if exception.response['Error']['Code'] == "404":
-                # The object does not exist.
-                print((
-                    "Unable to find object. Skipping the task, "
-                    "the following exception was found = {}".format(exception)
-                ))
-            else:
-                raise
+        data_stream = io.BytesIO(
+            json.dumps(json_data).encode("UTF-8")
+        )
 
-    def cache_lookup(self, data_stream: io.BytesIO, cache_key: str):
+        self.upload_s3_blob(
+            blob=data_stream,
+            s3_url=dst_url
+        )
+
+    def delete_cached_dataset(self, cache_key: str):
+        """deletes a cached dataset given a key
+
+        Args:
+            cache_key (str): dataset key
+        """
+        if self.redis_helper.redis_hexists(hashkey=cache_key) == 1:
+            print("Deleting cached dataset")
+            self.redis_helper.redis_delete(keys=[cache_key])
+
+    def cache_lookup(self, json_data: dict, cache_key: str):
         """cache lookup in elasticcache
 
         Args:
             cache_key (str): key to use when caching
             json_data (dict): data to cache which should be simple dictionary lookup
         """
-        json_data = json.loads(
-            data_stream.read().decode("UTF-8")
-        )
         if not isinstance(json_data, dict):
             raise Exception("ClientError: Json data is not a dictionary")
 
@@ -325,26 +349,42 @@ class DatasetStateManager:
                 print("Download dataset from prod/ in s3")
 
                 data_stream = self.download_s3_blob(s3_url=prod_url)
-
+                # load json data from bytes stream
+                json_data = json.loads(
+                    data_stream.read().decode("UTF-8")
+                )
                 print("Pushing to previous/ to s3")
                 self.execute_s3_versioning(
-                    data_stream=data_stream,
+                    json_data=json_data,
                     dst_url=previous_url
                 )
 
                 print("Download dataset from train/ from s3")
                 # get data
                 data_stream = self.download_s3_blob(s3_url=identifier)
+                json_data = json.loads(
+                    data_stream.read().decode("UTF-8")
+                )
 
+                tmp_cache_clone = "{}-tmp-clone".format(cache_key)
+                print("Creating a dataset clone")
+                self.cache_lookup(
+                    json_data=json_data,
+                    cache_key=tmp_cache_clone
+                )
                 # cache to elastic cache
                 print("Caching new prod dataset")
                 self.cache_lookup(
-                    data_stream=data_stream,
+                    json_data=json_data,
                     cache_key=cache_key
                 )
+                print("Removing clone")
+                self.delete_cached_dataset(
+                    cache_key=tmp_cache_clone
+                )
                 print("Pushing dataset to prod/ in s3")
                 self.execute_s3_versioning(
-                    data_stream=data_stream,
+                    json_data=json_data,
                     dst_url=prod_url
                 )
 
