@@ -377,16 +377,12 @@ class MaintenanceManager:
                     .format(client_name=client.name)
                 )
 
-                new_solution_base_name = create_resource_name(
-                    context=self.service.slug,
-                    function=client.name, # to mainten order
-                    additional_details=[engine_slug, 'training']
-                )
-
                 # train a new model if should_create_version True
                 new_solution = self.solution_state_manager.update_state(
                     solution=solution,
-                    solution_base_name=new_solution_base_name
+                    client_code=client.code,
+                    service_slug=self.service.slug,
+                    engine_slug=engine_slug,
                 )
 
                 # get solution arn
@@ -517,7 +513,10 @@ class MaintenanceManager:
 
                 new_campaign = self.campaign_state_manager.update_state(
                     campaign=campaign,
-                    solution=solution
+                    solution=solution,
+                    client_code=client.code,
+                    service_slug=self.service.slug,
+                    engine_slug=engine_slug
                 )
 
                 print('Successfully started updating campaign: {arn}'.format(arn=arn))
